@@ -21,10 +21,26 @@
 
 #include <glib-object.h>
 #include <babl/babl.h>
-#include <gegl-matrix.h>
-#include <gegl-types.h>
+#include "gegl-buffer-matrix2.h"
+#include "gegl-buffer-enums.h"
+
+#define GEGL_AUTO_ROWSTRIDE 0
 
 G_BEGIN_DECLS
+typedef struct _GeglTileBackend GeglTileBackend;
+typedef struct _GeglBuffer  GeglBuffer;
+typedef struct _GeglSampler       GeglSampler;
+
+typedef struct _GeglRectangle GeglRectangle;
+
+struct _GeglRectangle
+{
+  gint x;
+  gint y;
+  gint width;
+  gint height;
+};
+
 
 /***
  * GeglBuffer:
@@ -283,17 +299,21 @@ void            gegl_buffer_set               (GeglBuffer          *buffer,
                                                gint                 rowstride);
 
 
+
 /**
- * gegl_buffer_set_color:
+ * gegl_buffer_set_color_from_pixel:
  * @buffer: a #GeglBuffer
  * @rect: a rectangular region to fill with a color.
- * @color: the GeglColor to fill with.
+ * @pixel: pointer to the data of a single pixel
+ * @pixel_format: the babl format of the pixel, if missing - the soft format of dst.
  *
- * Sets the region covered by rect to the specified color.
+ * Sets the region covered by rect to the the provided pixel.
  */
-void            gegl_buffer_set_color         (GeglBuffer          *buffer,
-                                               const GeglRectangle *rect,
-                                               GeglColor           *color);
+void
+gegl_buffer_set_color_from_pixel (GeglBuffer          *buffer,
+                                  const GeglRectangle *rect,
+                                  const guchar        *pixel,
+                                  const Babl          *pixel_format);
 
 
 /**
@@ -422,15 +442,15 @@ GeglBuffer *    gegl_buffer_dup               (GeglBuffer       *buffer);
  * is more efficient.
  */
 
-void              gegl_buffer_sample_at_level (GeglBuffer       *buffer,
-                                               gdouble           x,
-                                               gdouble           y,
-                                               GeglMatrix2      *scale,
-                                               gpointer          dest,
-                                               const Babl       *format,
-                                               gint              level,
-                                               GeglSamplerType   sampler_type,
-                                               GeglAbyssPolicy   repeat_mode);
+void              gegl_buffer_sample_at_level (GeglBuffer        *buffer,
+                                               gdouble            x,
+                                               gdouble            y,
+                                               GeglBufferMatrix2 *scale,
+                                               gpointer           dest,
+                                               const Babl        *format,
+                                               gint               level,
+                                               GeglSamplerType    sampler_type,
+                                               GeglAbyssPolicy    repeat_mode);
 
 /**
  * gegl_buffer_sample: (skip)
@@ -458,14 +478,14 @@ void              gegl_buffer_sample_at_level (GeglBuffer       *buffer,
  * gegl_buffer_sampler_new() to create a sampler object instead, which is more
  * efficient.
  */
-void              gegl_buffer_sample          (GeglBuffer       *buffer,
-                                               gdouble           x,
-                                               gdouble           y,
-                                               GeglMatrix2      *scale,
-                                               gpointer          dest,
-                                               const Babl       *format,
-                                               GeglSamplerType   sampler_type,
-                                               GeglAbyssPolicy   repeat_mode);
+void              gegl_buffer_sample          (GeglBuffer        *buffer,
+                                               gdouble            x,
+                                               gdouble            y,
+                                               GeglBufferMatrix2 *scale,
+                                               gpointer           dest,
+                                               const Babl        *format,
+                                               GeglSamplerType    sampler_type,
+                                               GeglAbyssPolicy    repeat_mode);
 
 
 
@@ -481,12 +501,12 @@ void              gegl_buffer_sample          (GeglBuffer       *buffer,
 G_DEPRECATED
 void            gegl_buffer_sample_cleanup    (GeglBuffer *buffer);
 
-typedef void (*GeglSamplerGetFun)  (GeglSampler     *self,
-                                    gdouble          x,
-                                    gdouble          y,
-                                    GeglMatrix2     *scale,
-                                    void            *output,
-                                    GeglAbyssPolicy  repeat_mode);
+typedef void (*GeglSamplerGetFun)  (GeglSampler       *self,
+                                    gdouble            x,
+                                    gdouble            y,
+                                    GeglBufferMatrix2 *scale,
+                                    void              *output,
+                                    GeglAbyssPolicy    repeat_mode);
 
 /**
  * gegl_sampler_get_fun: (skip)
@@ -553,12 +573,12 @@ GeglSampler *    gegl_buffer_sampler_new_at_level (GeglBuffer       *buffer,
  *
  * Perform a sampling with the provided @sampler.
  */
-void              gegl_sampler_get            (GeglSampler    *sampler,
-                                               gdouble         x,
-                                               gdouble         y,
-                                               GeglMatrix2    *scale,
-                                               void           *output,
-                                               GeglAbyssPolicy repeat_mode);
+void              gegl_sampler_get            (GeglSampler       *sampler,
+                                               gdouble            x,
+                                               gdouble            y,
+                                               GeglBufferMatrix2 *scale,
+                                               void              *output,
+                                               GeglAbyssPolicy   repeat_mode);
 
 /* code template utility, updates the jacobian matrix using
  * a user defined mapping function for displacement, example
@@ -699,7 +719,81 @@ glong gegl_buffer_signal_connect (GeglBuffer *buffer,
                                   GCallback   c_handler,
                                   gpointer    data);
 
+
+
+/***
+ * Aligned memory:
+ *
+ * GEGL provides functions to allocate and free buffers that are guaranteed to
+ * be on 16 byte aligned memory addresses.
+ */
+
+/**
+ * gegl_malloc: (skip)
+ * @n_bytes: the number of bytes to allocte.
+ *
+ * Allocates @n_bytes of memory. If n_bytes is 0 it returns NULL.
+ *
+ * Returns a pointer to the allocated memory.
+ */
+gpointer gegl_malloc                  (gsize    n_bytes) G_GNUC_MALLOC;
+
+/**
+ * gegl_free: (skip)
+ * @mem: the memory to free.
+ *
+ * Frees the memory pointed to by @mem, if @mem is NULL it will warn and abort.
+ */
+void     gegl_free                    (gpointer mem);
+
+/**
+ * gegl_calloc: (skip)
+ * @size: size of items to allocate
+ * @n_memb: number of members
+ *
+ * allocated 0'd memory.
+ */
+gpointer gegl_calloc (gsize size, int n_memb) G_GNUC_MALLOC;
+
+/**
+ * gegl_memset_pattern: (skip)
+ * @dst_ptr: pointer to copy to
+ * @src_ptr: pointer to copy from
+ * @pattern_size: the length of @src_ptr
+ * @count: number of copies
+ *
+ * Fill @dst_ptr with @count copies of the bytes in @src_ptr.
+ */
+void gegl_memset_pattern              (void *       dst_ptr,
+                                              const void * src_ptr,
+                                              gint         pattern_size,
+                                              gint         count);
+
+
+/**
+ * gegl_buffer_flush_ext:
+ * @buffer: a GeglBuffer
+ * @rect: rectangle
+ *
+ * Invokes the external flush function, if any is set on the provided buffer -
+ * this ensures that data pending - in the current implementation only OpenCL -
+ * externally to be synchronized with the buffer. Multi threaded code should
+ * call such a synchronization before branching out to avoid each of the
+ * threads having an implicit synchronization of its own.
+ */
+void
+gegl_buffer_flush_ext (GeglBuffer *buffer, const GeglRectangle *rect);
+
 #include <gegl-buffer-iterator.h>
+#include <gegl-rectangle.h>
+
+
+GType gegl_buffer_get_type  (void) G_GNUC_CONST;
+#define GEGL_TYPE_BUFFER    (gegl_buffer_get_type ())
+#define GEGL_BUFFER(obj)    (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEGL_TYPE_BUFFER, GeglBuffer))
+#define GEGL_IS_BUFFER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEGL_TYPE_BUFFER))
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (GeglBuffer, g_object_unref)
+
 
 G_END_DECLS
 #endif
